/* Copyright (c) 2013 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 */

#include "bootloader_util.h"
#include <stdint.h>
#include "nordic_common.h"
#include "bootloader_types.h"
#include "dfu_types.h"

#if __GNUC__ && __ARM_EABI__

__attribute__ ((section(".bootloaderSettings"))) uint8_t m_boot_settings[1024] ;
__attribute__ ((section(".uicrBootStartAddress"))) uint32_t m_uicr_bootloader_start_address = BOOTLOADER_REGION_START;
const bootloader_settings_t const * const mp_bootloader_settings = (bootloader_settings_t *) &m_boot_settings[0];   /**< Read only pointer to bootloader settings in flash. */


static inline void StartApplication(uint32_t start_addr)
{
    __asm volatile(
		".equ MASK_ONES, 0xFFFFFFFF\n\t"
		".equ	MASK_ZEROS, 0x00000000\n\t"
		".equ	xPSR_RESET, 0x21000000\n\t"
		".equ	EXC_RETURN_CMD, 0xFFFFFFF9\n\t"
		"LDR   R5, [R0]\n\t"
		"MSR   MSP, R5\n\t"
		"LDR   R6, [R0, #0x04]\n\t"
		"LDR   R2, =MASK_ZEROS\n\t"
		"MRS   R3, IPSR\n\t"
		"CMP   R2, R3\n\t"
		"MOV   R0, R6\n\t"
		"BNE   isr_abort\n\t"
		"LDR   R4, =MASK_ONES\n\t"
		"MOV   LR, R4\n\t"
		"BX    R6\n\t"
		"isr_abort:\n\t"
		"LDR   R4,=MASK_ONES\n\t"
		"LDR   R5,=MASK_ONES\n\t"
		"MOV   R6, R0\n\t"
		"LDR   R7,=xPSR_RESET\n\t"
		"PUSH  {r4-r7}\n\t"
		"LDR   R4,=MASK_ZEROS\n\t"
		"LDR   R5,=MASK_ZEROS\n\t"
		"LDR   R6,=MASK_ZEROS\n\t"
		"LDR   R7,=MASK_ZEROS\n\t"
		"PUSH  {r4-r7}\n\t"
		"LDR   R0,=EXC_RETURN_CMD\n\t"
		"BX    R0\n\t"
          ".ALIGN\n\t");
}
#endif


void bootloader_util_app_start(uint32_t start_addr)
{
    StartApplication(start_addr);
}


void bootloader_util_settings_get(const bootloader_settings_t ** pp_bootloader_settings)
{
    *pp_bootloader_settings = mp_bootloader_settings;
}
